﻿/*MyPhotoIndex Free Photo Organizer <www.myphotoindex.com>
  Copyright (C) <2006-2010>  <Asaf Yarkoni asaf@yarkoni.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
using System;
using System.Collections.Generic;
using System.Text;
using MyPhotoIndex.Interfaces.Plugins;
using System.Collections.ObjectModel;
using MyPhotoIndex.Interfaces;
using MyPhotoIndex.Controls;
using MyPhotoIndex.Plugins.Histogram.Properties;

namespace MyPhotoIndex.Plugins.Histogram
{
    public class Histogram : IUtilityPlugin
    {
        private IApplicationContext m_applicationContext;
        static HistogramForm ms_histogramForm = null;

        #region IUtilityPlugin Members

        public void ImageSelectionChanged(ReadOnlyCollection<int> selectedImages, SelectedImageInfo lastSelected)
        {
            if (ms_histogramForm != null &&
                ms_histogramForm.Visible == true)
            {
                ms_histogramForm.SelectionChanged(selectedImages);
            }
        }

        public void RegisterMenues(List<IPluginMenuInfo> pluginMenuInfoList)
        {
            PluginMenuInfo menuInfo = new PluginMenuInfo();
            menuInfo.MenuItemId = Guid.NewGuid();
            menuInfo.Name = "Show histogram";
            menuInfo.Icon = Resources.column_chart;
            menuInfo.MenuItemClick += new EventHandler<PluginMenuClickEventArgs>(menuInfo_MenuItemClick);
            menuInfo.MenuItemQueryState += new EventHandler<PluginMenuStateQueryEventArgs>(menuInfo_MenuItemQueryState);

            pluginMenuInfoList.Add(menuInfo);
        }

        void menuInfo_MenuItemQueryState(object sender, PluginMenuStateQueryEventArgs e)
        {
            e.Enabled = ms_histogramForm == null || (ms_histogramForm.Visible == false);
        }

        void menuInfo_MenuItemClick(object sender, PluginMenuClickEventArgs e)
        {
            if (ms_histogramForm == null)
             {
                 ms_histogramForm = new HistogramForm(m_applicationContext);
             }

            ms_histogramForm.Show(ApplicationAlert.DefaultOwner);
            ms_histogramForm.SelectionChanged(e.SelectedImages);
        }

        #endregion

        #region IPlugin Members

        public string Description
        {
            get { return "Histogram"; }
        }

        public bool Load(IApplicationContext applicationContext)
        {
            m_applicationContext = applicationContext;

            return true;
        }

        public string Name
        {
            get { return "Histogram"; }
        }

        public void Unload()
        {
            if (ms_histogramForm != null)
            {
                ms_histogramForm.CloseForm();
            }
        }

        #endregion
    }
}
