/*MyPhotoIndex Free Photo Organizer <www.myphotoindex.com>
  Copyright (C) <2006-2007>  <Asaf Yarkoni asaf@yarkoni.net>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
using System;
using System.Collections.Generic;
using System.Text;
using MyPhotoIndex.Interfaces.Plugins;
using System.Drawing.Imaging;
using System.Drawing;

namespace MyPhotoIndex.Plugins.SlideShowEffects
{
    public class WipeRightEffect : ISlideShowEffectPlugin
    {
        #region ISlideShowEffectPlugin Members

        public System.Drawing.Image Icon
        {
            get { return null; }
        }

        public void Transform(Graphics graphics, Rectangle drawingBounds, Image oldImage, Image newImage, Rectangle oldBounds, Rectangle newBounds)
        {
            int partSize = 5;

            Rectangle sliceRect = new Rectangle(newBounds.X, newBounds.Y, partSize, newBounds.Height);

            using (Region exRegion = new Region(drawingBounds))
            {
                exRegion.Exclude(newBounds);
                graphics.Clip = exRegion;

                using (Brush brush = new SolidBrush(Color.FromArgb(64, 64, 64)))
                {
                    graphics.FillRectangle(brush, drawingBounds);
                }
            }

            using (Region fullRegion = new Region(drawingBounds))
            {
                graphics.Clip = fullRegion;
                for (int slice = 0; slice < newBounds.Width; slice += partSize)
                {
                    sliceRect.X = newBounds.X + slice;
                    graphics.DrawImage(newImage, sliceRect, slice, 0, partSize, newImage.Height, GraphicsUnit.Pixel);
                }
            }
        }

        #endregion

        #region IPlugin Members

        public string Description
        {
            get { return "Wipe right effect"; }
        }

        public bool Load(MyPhotoIndex.Interfaces.IApplicationContext applicationContext)
        {
            return true;
        }

        public string Name
        {
            get { return "Wipe right effect"; }
        }

        public void Unload()
        {

        }

        #endregion

        public override string ToString()
        {
            return Name;
        }
    }
}

